import os
import sys
import time
import struct
import glob
import pickle

from subprocess import call
from bluepy import btle
from bluepy.btle import Scanner, DefaultDelegate

BLOCK_SIZE = 18

GALAXY_SERVICE_UUID = btle.UUID('05040302-0102-0304-0506-070807060504')
REBOOT_CHARACTERISTIC_UUID = btle.UUID('08070605-0403-0201-0102-030405060712')

def getCharacteristic(service, characteristicUuid):
    result = service.getCharacteristics(characteristicUuid)
    if len(result) > 0:
        return result[0]

class Spu:
    def __init__(self, dev):
        self.peripheral = None
        self.__dev = dev
        self.resent = False

    def getName(self):
        return self.__dev.getValueText(0x09)

    def getAddress(self):
        return self.__dev.addr

    def getRssi(self):
        return self.__dev.rssi

    def reboot(self):
        self.__rebootCharacteristic.write('\x01', withResponse=True)

    def connect(self):
        if self.peripheral:
            return
        print 'Connecting to %s (%s)' % (self.getName(), self.__dev.addr)
        self.peripheral = btle.Peripheral(self.__dev)
        self.__retrieveCharacteristics()
        print 'Connected'

    def disconnect(self):
        self.peripheral.disconnect()
        self.peripheral = None
	print 'Rebooted!'

    def __retrieveCharacteristics(self):
        service = self.peripheral.getServiceByUUID(GALAXY_SERVICE_UUID)
        self.__rebootCharacteristic = getCharacteristic(service, REBOOT_CHARACTERISTIC_UUID)

def isSpu(dev):
    return dev.getValueText(0x07) == str(GALAXY_SERVICE_UUID)

def scanSpus():
    print 'Scanning SPUs'
    scanner = Scanner()
    devices = scanner.scan(3)
    return [Spu(dev) for dev in devices if isSpu(dev)]

def displayChoices(spus):
    for i, spu in enumerate(spus):
        print "%d. %s %s RSSI=%d" % (i + 1, spu.getName().ljust(20), spu.getAddress(), spu.getRssi())

def selectSpus(spus):
    userInput = raw_input("SPUs to reboot (separate with spaces, leave empty for all):\r\n")
    if userInput == '':
        return spus
    spuIndexes = map(lambda i: int(i) - 1, userInput.split(' '))
    return [spus[i] for i in spuIndexes if i in range(len(spus))]

def launchReboot(rebootAll):
    spus = scanSpus()
    if rebootAll:
        spusToReboot = spus
    else:
        displayChoices(spus)
        spusToReboot = selectSpus(spus)
    for spu in spusToReboot:
        rebootSpu(spu, rebootAll)

def rebootSpu(spu, force):
    spu.connect()
    spu.reboot()
    spu.disconnect()

def loadSpus(dict):
    with open(dict, 'rb') as config_dictionary_file:
        return pickle.load(config_dictionary_file)

def printHelp():
    print "usage: sudo python", sys.argv[0], "[-y] [help]"
    print "    -y      reboot all SPUs (does not prompt the user)"
    print "    help    print this help"

if __name__ == "__main__":
    if "help" in sys.argv[1:]:
        printHelp()
    else:
        updateAll = False
        for arg in sys.argv[1:]: 
		# print arg
		if arg == 'reboot':
                	spus = loadSpus('spu.dictionary')
                	# path = sys.argv[2]  # firmware file and index of spu to upload
                	num = int(sys.argv[2])
			rebootSpu(spus[num-1],'y')


